const { isEmpty, isObject } = require('./judgement');

/**
 * 将json转换为数组，数组元素为对象的每个key的value
 * @param {Object} json json数据
 * @returns {Array} 转换得到的数组
 */
module.exports.jsonToEnums = json => Object.keys(json).map(key => json[key]);

/**
 * 将json转换为数组，数组元素为对象的key并以其对应的value排序
 * @param {Object} json json数据
 * @returns {Array} 转换得到的数组
 */
module.exports.jsonToKeyMap = json => Object.keys(json).sort((v1, v2) => json[v1] - json[v2]);

/**
 * 将json转换为数组，数组元素为{key, value, queue}
 * 其中key为json的key，value为key对应的值，queue为value中的queue，默认为0
 * 并以queue值排序
 * @param {Object} json json数据
 * @returns {Array} 转换得到的数组
 */
module.exports.jsonToArray = json => Object.keys(json).map(key => {
  const ele = {};
  ele.key = key;
  ele.value = json[key];
  ele.queue = json[key].queue || 0;
  return ele;
}).sort((v1, v2) => v1.queue - v2.queue);

/**
 * 对目标字符串中间一段字串使用stuff进行替换，隐藏敏感信息
 * @param {string} str 要转换的字符串
 * @param {number} start 开始位置
 * @param {number} length 需要替换的长度
 * @param {string} stuff 替换字符
 * @returns {string} 转换后的字符串
 */
module.exports.mask = (str, start, length, stuff = '*') => {
  const arr = str.split('');
  arr.splice(start, length, ...new Array(length).fill(stuff));
  return arr.join('');
};

/**
 * 数字截位
 * @param {number} num 数字
 * @param {number} n 需要补到多少位
 * @returns {string} 截位后的数字字符串
 */
module.exports.pad = (num, n) => `${new Array(n).fill('0').join('')}${num}`.slice(-n);

/**
 * 将参数对象转换为序列化字符串
 * @param {object} params 参数对象
 * @param {boolean} encode 是否对key和value进行encodeURI操作
 * @param {boolean} retainEmpty 是否保留值为空的参数
 * @param {Function} compareFn key的排序方法
 * @returns {string} 序列化后的字符串
 */
module.exports.parseParams = (params, encode = true, retainEmpty = false, compareFn) => Object.keys(params).sort(compareFn).reduce((memo, key) => {
  let value = params[key];
  if (!retainEmpty && isEmpty(value)) return memo;
  if (isObject(value)) value = JSON.stringify(value);
  if (encode) {
    value = encodeURIComponent(value);
    key = encodeURIComponent(key);
  }
  memo.push([key, value].join('='));
  return memo;
}, []).join('&');

const cityCode = require('./cityCode.json');
module.exports.parseIdCardNo = idCardNo => {
  const code = idCardNo.slice(0, 6);
  const city = cityCode[code];
  const birthYear = idCardNo.slice(6, 10);
  const birthMonth = idCardNo.slice(10, 12);
  const birthDay = idCardNo.slice(12, 14);
  const birth = new Date(`${birthYear}/${birthMonth}/${birthDay}`);
  const genderCode = idCardNo.slice(-2, -1);
  const gender = ['女', '男'][parseInt(genderCode) % 2];
  return { city, birth, gender };
};

module.exports.jsonToMap = (json = {}) => Object.keys(json).reduce((memo, key) => {
  memo[json[key]] = key;
  return memo;
}, {});